#include "tdd_file_transfer.h"
#include "tal_memory.h"

#define TUYA_SKD_DBG    1
#if defined (TUYA_SKD_DBG)
#include "tkl_uart.h"
#include "tlprintf.h"
#define app_transfer_debug(...)   tkl_dbg_printf(__VA_ARGS__)
#else
#define app_transfer_debug(...)
#endif

#define TRANSFER_NEED_STOP_REC          0x80
#define TRANSFER_NEED_STOP_SEND         0x40

typedef enum {
    WHOLE_TYPE = 0,
    SUBSTEP_TYPE,
}MULIPLE_PACKET_TYPE_T;

typedef enum{
    CMD_START = 0,
    CMD_DOING,
    CMD_END,
}MULTIPLE_PACKET_TRANSFER_CMD_TYPE_T;

typedef enum {
    REC_TRANSFER_IDLE = 0,
    REC_TRANSFER_START, 
    REC_TRANSFER_DOING,
}MULTIPLE_PACKET_TRANSFER_REC_STATUS_T;

typedef enum {
    SEND_TRANSFER_IDLE = 0,
    SEND_TRANSFER_START,
    SEND_TRANSFER_DOING,
}MULTIPLE_PACKET_TRANSFER_SEND_STATUS_T;

typedef enum {
    REC_BUSY = 0,
    REC_STOP_REPLACE_REC = (0x01|TRANSFER_NEED_STOP_REC),/*停止当前的接收，先接收优先级更高的命令数据，将当前的数据push进链表，如红外同步时先进行控制命令*/
    REC_STOP_REPLACE_SEND = (0x02|TRANSFER_NEED_STOP_REC),/*停止当前的接收，优先进行发送，将当前的数据push进链表，如红外同步时进行学习码上报命令*/
    REC_IDLE = 0x03,
    SEND_BUSY = 0x04,
    SEND_STOP_REPLACE_REC = (0x05|TRANSFER_NEED_STOP_SEND),/*停止当前的发送，先接收优先级更高的命令数据，将当前的数据push进链表*/
    SEND_STOP_REPLACE_SEND = (0x06|TRANSFER_NEED_STOP_SEND),/*停止当前的发送，优先进行发送优先级更高的命令数据，将当前的数据push进链表*/
    SEND_IDLE = 0x07,
    REC_SEND_STOP_REPLACE_REC = (0x08|TRANSFER_NEED_STOP_REC|TRANSFER_NEED_STOP_SEND),/*优先级一样的接收和发送被同时打断，优先进行传入的接收命令*/
    REC_SEND_STOP_REPLACE_SEND = (0x09|TRANSFER_NEED_STOP_REC|TRANSFER_NEED_STOP_SEND),/*优先级一样的接收和发送被同时打断，优先进行传入的发送命令*/
}TRANSFER_STATUS_T;

typedef struct {
    MULIPLE_PACKET_TYPE_T type;
    MULTIPLE_PACKET_TRANSFER_REC_STATUS_T status;
    start_notification_pdu_t notify;
    UINT8_T* buf;
    UINT16_T depth;/*主要的作用是为了分包分步过程中判断数据是否超过接受数组上限需要进行一次分步回调*/
    UINT32_T offset;
}multiple_packet_transfer_rec_info_t;

typedef struct {
    MULIPLE_PACKET_TYPE_T type;
    MULTIPLE_PACKET_TRANSFER_SEND_STATUS_T status;
    start_notification_pdu_t notify;
    UINT8_T* buf;
    UINT8_T packet_size;/*获取的一帧数据长度,该数据为完全的待发送数据长度，而不是加上了格式的DEF_BLOCK_REQ_MAX_SIZE*/
    UINT16_T depth;
    UINT32_T offset;
}multiple_packet_transfer_send_info_t;

typedef struct{
    UINT16_T addr;
    UINT32_T file_id;
    multiple_packet_transfer_rec_info_t info;
}run_rec_info_t;

typedef struct{
    UINT16_T addr;
    UINT32_T file_id;
    multiple_packet_transfer_send_info_t info;
}run_send_info_t;

typedef struct{
    run_rec_info_t rec;
    run_send_info_t send;
}run_info_t;

typedef struct {
    bool_t doing;
    UINT16_T addr;
    start_notification_pdu_t pdu;  
}user_flash_start_notify_t;

typedef struct parallel_config_t{
    UINT8_T cmd_type;
    bool_t delete_after_push;
    union{
        run_rec_info_t rec;
        run_send_info_t send;
    }info;
    struct parallel_config_t  *next;
}parallel_transfer_config_t;/*并行控制状态信息*/

static run_info_t g_run_info = {
    .rec.info.buf = NULL,
    .send.info.buf = NULL,
};

static UINT16_T g_muliple_packet_send_file_id = 0;
static parallel_transfer_config_t g_file_head;

bool_t g_need_refresh_flag = FALSE;
UINT8_T* g_muliple_packet_rec_buf_head = NULL;/*用于暂存4字节的数据长度，注意如果msg_rec调用时数据长度超过0xff则数据的前4字节表示长度*/

static TIMER_ID sw_packet_callback_use = 0;/*延时调用应用层用于提前发送end request，避免应用层while(1)类型操作导致end request发不出去*/

REC_MULIPLE_PACKET_ERROR_CODE_T app_rec_start_notify_callback(UINT16_T addr,start_notification_pdu_t pdu);
static bool_t refresh_transfer_rec_info(bool_t callback,bool_t result);
static bool_t refresh_transfer_send_info(bool_t callback,bool_t result);
static void hal_multiple_packet_rec_evt_handle(TIMER_ID timer_id, VOID_T *arg);
void ext_set_new_frame_flag(bool_t refresh_flag);

bool_t  push_transfer_list(parallel_transfer_config_t input);
bool_t  get_transfer_list(parallel_transfer_config_t** output);
bool_t  pop_transfer_list(void);
TRANSFER_STATUS_T check_stop_transfering_file(start_notification_pdu_t pdu,bool_t rec);
bool_t check_fileid_send_end_request(UINT16_T file_id,END_RESPOND_STATUS_T end_status);

/*************************************************************************************************
 * 
 * 
 *                                    分包公共部分
 * 
 * 
 * **************************************************************************************************/

/**
 * @description: 确认当前的传输过程性质用于确认优先级排序
 * @param {type} none
 * @return: none
 */
TUYA_WEAK_ATTRIBUTE UINT8_T judge_transfer_cmd_type(start_notification_pdu_t pdu,bool_t rec)
{
    UINT8_T cmd_type = 0;

    if(rec){
        cmd_type = 0x00;
    }
    else{
        cmd_type = 0x10;
    }

    switch(pdu.cluster_id)
    {
#if 0 /*给予参考用于区分优先级*/
        case CLUSTER_TUYA_IR_CONTROL_CLUSTER_ID:/*红外cluster*/
            if(ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER == pdu.cmd_type){
                switch(pdu.cmd)
                {
                    case CMD_IR_STUDY_STATUS_COMMAND_ID:
                    case CMD_IR_KEYID_CONTROL_COMMAND_ID:
                    case CMD_IR_KEYCODE_SEND_COMMAND_ID:
                    case CMD_IR_DELETE_KEYCODE_COMMAND_ID:
                        cmd_type = REC_IR_CONTROL;
                    break;

                    case CMD_IR_KEYCODE_REPLACE_COMMAND_ID:  
                    case CMD_IR_KEYCODE_IMMEDIATE_DOWNLOAD_COMMAND_ID:
                    case CMD_IR_KEYCODE_DELAY_DOWNLOAD_COMMAND_ID:
                        cmd_type = REC_IR_SYNC;
                    break;

                    case CMD_IR_STUDY_KEYCODE_COMMAND_ID:
                    case CMD_IR_REPORT_KEYID_COMMAND_ID:
                        cmd_type = SEND_IR_STUDY_REPORT;
                    break;

                    default:
                    break;
                }
            }
        break;
#endif
        default:
        break;
    }
    return cmd_type;
}



/*发送函数的结果回调函数，用于no sleep，sleep设备之间切换时保存数据使用*/
TUYA_WEAK_ATTRIBUTE void app_send_data_result_callback(TAL_SEND_ST_E result, TAL_ZG_SEND_DATA_T * data)
{
    app_transfer_debug("send result: %d\r\n",result);
    if(SEND_ST_OK == result)
    {
        if((SEND_MODE_DEV==data->addr.mode)  && (CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID==data->addr.type.dev.cluster_id)) 
        {
            if(ZG_ZCL_DATA_CLIENT_TO_SERVER==data->direction){
                switch(data->command_id){
                    case CMD_TRANSFER_START_RESPOND_COMMAND_ID:/*对方收到了我的notify start responds*/
                    break;
                    case CMD_TRANSFER_BLOCK_REQUEST_COMMAND_ID:
                    break;
                    case CMD_TRANSFER_END_REQUEST_COMMAND_ID:
                    break;  
                    default:
                    break;                  
                }
            }
            else if(ZG_ZCL_DATA_SERVER_TO_CLIENT==data->direction){
                switch(data->command_id){
                    case CMD_TRANSFER_START_NOTIFICATION_COMMAND_ID:
                    break;
                    default:
                    break;
                }
            }
        }
    }
    else 
    {
        if((SEND_MODE_DEV==data->addr.mode)  && (CLUSTER_MULTIPLE_PACKET_TRANSFER_CLUSTER_ID==data->addr.type.dev.cluster_id)) 
        {
            if(ZG_ZCL_DATA_CLIENT_TO_SERVER==data->direction){
                switch(data->command_id){
                    case CMD_TRANSFER_START_RESPOND_COMMAND_ID:/*对方收到了我的notify start responds*/
                    break;
                    case CMD_TRANSFER_BLOCK_REQUEST_COMMAND_ID:
                    break;
                    case CMD_TRANSFER_END_REQUEST_COMMAND_ID:
                    break;  
                    default:
                    break;                  
                }
            }
            else if(ZG_ZCL_DATA_SERVER_TO_CLIENT==data->direction){
                switch(data->command_id){
                    case CMD_TRANSFER_START_NOTIFICATION_COMMAND_ID:
                    break;
                    default:
                    break;
                }
            }
        }
    }
}



/*给予参考用于确认是否可以进行分包进程*/
TUYA_WEAK_ATTRIBUTE bool_t check_start_notification_status(UINT16_T cluster_id,UINT8_T cmd_type,UINT8_T cmd,UINT32_T total_size)
{
    bool_t result = TRUE;
#if 0
    if((CLUSTER_TUYA_IR_CONTROL_CLUSTER_ID==cluster_id)&&(ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER==cmd_type)\
    &&(CMD_IR_KEYCODE_IMMEDIATE_DOWNLOAD_COMMAND_ID==cmd))
    {
        if(IR_FLASH_4K_BLOCK_MAX_SIZE<total_size){
            result = FALSE;
        }
        else if(FALSE==check_download_status(total_size)){
            result = FALSE;
        }         
    }
#endif
    return result;
}


/**
 * @description: 管理传输数据
 * @param {type} none
 * @return: TRUE 表示有数据切换,FALSE 表示没有数据切换
 */
TUYA_WEAK_ATTRIBUTE bool_t manage_transfering_file(start_notification_pdu_t pdu,bool_t rec_start,bool_t rec_end,bool_t send_start,bool_t send_end)
{
    bool_t result = FALSE;
    TRANSFER_STATUS_T start_control = REC_BUSY;
    parallel_transfer_config_t *push_node;
    parallel_transfer_config_t *pop_node;
    parallel_transfer_config_t *temp_node;

    if(1==(rec_start+rec_end+send_start+send_end)){
        if(rec_start || send_start){
            if(rec_start){
                start_control = check_stop_transfering_file(pdu,TRUE);
            }
            else if(send_start){
                start_control = check_stop_transfering_file(pdu,FALSE);
            }
            if((start_control&TRANSFER_NEED_STOP_REC) || (start_control&TRANSFER_NEED_STOP_SEND)){
                push_node = tal_malloc(sizeof(parallel_transfer_config_t));
                if(push_node){
                    if(start_control&TRANSFER_NEED_STOP_REC){
                        push_node->cmd_type = judge_transfer_cmd_type(g_run_info.rec.info.notify,TRUE);
                        push_node->delete_after_push = FALSE;
                        push_node->next = NULL;
                        tal_system_memcpy((UINT8_T*)&push_node->info.rec,(UINT8_T*)&g_run_info.rec,sizeof(run_rec_info_t));
                        if(SUBSTEP_TYPE == push_node->info.rec.info.type){
                            if(TRUE==tal_sw_timer_is_running(sw_packet_callback_use)){/*在等待回调的过程中且恰好会被2K整除*/                                
                                if((push_node->info.rec.info.offset/push_node->info.rec.info.depth) && ((push_node->info.rec.info.offset%push_node->info.rec.info.depth)<DEF_BLOCK_REQ_MAX_SIZE)){
                                    push_node->info.rec.info.offset = ((push_node->info.rec.info.offset/push_node->info.rec.info.depth)-1)*push_node->info.rec.info.depth;
                                    app_transfer_debug("offset = 2K*n\r\n");
                                }                                                                    
                            }
                            else{
                                push_node->info.rec.info.offset = (push_node->info.rec.info.offset/push_node->info.rec.info.depth)*push_node->info.rec.info.depth;
                                app_transfer_debug("offset store 2k\r\n");
                            }
                        }
                        else if(WHOLE_TYPE == push_node->info.rec.info.type){
                            push_node->info.rec.info.offset = 0;
                        }

                        app_transfer_debug("push rec into transfer list \r\n");
                        app_transfer_debug("cmd_type : %x , addr : %02x , file_id : %04x  \r\n",\
                                    push_node->cmd_type,\
                                    push_node->info.rec.addr,\
                                    push_node->info.rec.file_id);
                        app_transfer_debug("type : %x , status : %x , buf : 0x%04x , depth : %d , offset :  %d \r\n",\
                                    push_node->info.rec.info.type,\
                                    push_node->info.rec.info.status,\
                                    push_node->info.rec.info.buf,\
                                    push_node->info.rec.info.depth,\
                                    push_node->info.rec.info.offset);
                        app_transfer_debug("notify file_id : %02x , notify total_size : %d , notify random_time : %04x , notify cluster_id : 0x%02x , notify cmd_type :  0x%x , notify cmd :  0x%x , notify reserved :  %02x\r\n",\
                                    push_node->info.rec.info.notify.file_id,\
                                    push_node->info.rec.info.notify.total_size,\
                                    push_node->info.rec.info.notify.random_time,\
                                    push_node->info.rec.info.notify.cluster_id,\
                                    push_node->info.rec.info.notify.cmd_type,\
                                    push_node->info.rec.info.notify.cmd,\
                                    push_node->info.rec.info.notify.reserved);

                        push_transfer_list(*push_node);
                        refresh_transfer_rec_info(FALSE,FALSE);
                        result = TRUE;
                    }
                    if(start_control&TRANSFER_NEED_STOP_SEND){
                        push_node->cmd_type =  judge_transfer_cmd_type(g_run_info.send.info.notify,FALSE);
                        push_node->delete_after_push = FALSE;
                        push_node->next = NULL;
                        tal_system_memcpy((UINT8_T*)&push_node->info.send,(UINT8_T*)&g_run_info.send,sizeof(run_send_info_t));
                        if(SUBSTEP_TYPE == push_node->info.send.info.type){
                            push_node->info.send.info.offset = (push_node->info.send.info.offset/push_node->info.rec.info.depth)*push_node->info.rec.info.depth;
                        }
                        else if(WHOLE_TYPE == push_node->info.send.info.type){
                            push_node->info.send.info.offset = 0;
                        }

                        app_transfer_debug("push send into transfer list \r\n");
                        app_transfer_debug("cmd_type : %x , addr : %02x , file_id : %04x  \r\n",\
                                    push_node->cmd_type,\
                                    push_node->info.send.addr,\
                                    push_node->info.send.file_id);
                        app_transfer_debug("type : %x , status : %x , buf : 0x%04x , depth : %d , offset :  %d \r\n",\
                                    push_node->info.send.info.type,\
                                    push_node->info.send.info.status,\
                                    push_node->info.send.info.buf,\
                                    push_node->info.send.info.depth,\
                                    push_node->info.send.info.offset);
                        app_transfer_debug("notify file_id : %02x , notify total_size : %d , notify random_time : %04x , notify cluster_id : 0x%02x , notify cmd_type :  0x%x , notify cmd :  0x%x , notify reserved :  %02x\r\n",\
                                    push_node->info.send.info.notify.file_id,\
                                    push_node->info.send.info.notify.total_size,\
                                    push_node->info.send.info.notify.random_time,\
                                    push_node->info.send.info.notify.cluster_id,\
                                    push_node->info.send.info.notify.cmd_type,\
                                    push_node->info.send.info.notify.cmd,\
                                    push_node->info.send.info.notify.reserved);

                        push_transfer_list(*push_node);
                        refresh_transfer_send_info(FALSE,FALSE);
                        result = TRUE;
                    }
                    tal_free(push_node);
                }
            }
            else{
                if((rec_start && (REC_IDLE == start_control)) || (send_start && (SEND_IDLE == start_control))){
                    result = TRUE;
                }
            }
        }
        else if(rec_end || send_end){
            pop_node = tal_malloc(sizeof(parallel_transfer_config_t));
            if(pop_node){
                temp_node = pop_node;/*保存tal_malloc的地址给保存起来，避免下面get后找不到tal_malloc的值了*/
                pop_node->next = NULL;
                if(rec_end || send_end){
                    if(get_transfer_list(&pop_node)){

                        app_transfer_debug("pop info from transfer list \r\n");
                        app_transfer_debug("cmd_type : %x , addr : %02x , file_id : %04x  \r\n",\
                                    pop_node->cmd_type,\
                                    pop_node->info.rec.addr,\
                                    pop_node->info.rec.file_id);
                        app_transfer_debug("type : %x , status : %x , buf : 0x%04x , depth : %d , offset :  %d \r\n",\
                                    pop_node->info.rec.info.type,\
                                    pop_node->info.rec.info.status,\
                                    pop_node->info.rec.info.buf,\
                                    pop_node->info.rec.info.depth,\
                                    pop_node->info.rec.info.offset);
                        app_transfer_debug("notify file_id : %02x , notify total_size : %d , notify random_time : %04x , notify cluster_id : 0x%02x , notify cmd_type :  0x%x , notify cmd :  0x%x , notify reserved :  %02x\r\n",\
                                    pop_node->info.rec.info.notify.file_id,\
                                    pop_node->info.rec.info.notify.total_size,\
                                    pop_node->info.rec.info.notify.random_time,\
                                    pop_node->info.rec.info.notify.cluster_id,\
                                    pop_node->info.rec.info.notify.cmd_type,\
                                    pop_node->info.rec.info.notify.cmd,\
                                    pop_node->info.rec.info.notify.reserved);

                        if(TRUE == pop_node->delete_after_push){
                            pop_node->delete_after_push = FALSE;

                            pop_transfer_list();
                            
                        }
                        else{
                            switch(pop_node->cmd_type){
                                case 0x00:
                                    /*继续原来的帧的接收*/
                                    tal_system_memcpy((UINT8_T*)&g_run_info.rec,(UINT8_T*)&pop_node->info.rec,sizeof(run_rec_info_t));
                                    g_run_info.rec.info.buf = (g_muliple_packet_rec_buf_head+sizeof(UINT32_T));
                                    tal_system_memset(g_run_info.rec.info.buf,0x00,g_run_info.rec.info.depth);
                                    pop_transfer_list();
                                    set_rec_wait_time(TRUE);
                                    ext_set_quick_time(TRUE,QUICK_POLL_SLICE_TIME);
                                    send_block_request_deal();
                                break;
                                case 0x10:
                                    /*继续原来的帧的发送*/
                                    /*TODO,发送的数据需要进行保存，便于中断后继续发送恢复*/
                                break;
                                default:
                                break;
                            } 
                        }                       
                    }
                } 
                tal_free(temp_node);
            }
        }
    }
    return result;
}



/**
 * @description: 将要发送的数据加到发送链表的最后,最后的表示优先级最高需要优先进行发送
 * @param {type} none
 * @return: none
 */
bool_t  push_transfer_list(parallel_transfer_config_t input)
{
    bool_t result = FALSE;
    parallel_transfer_config_t *node;
    parallel_transfer_config_t *head = &g_file_head;

    node = (parallel_transfer_config_t*)tal_malloc(sizeof(parallel_transfer_config_t));
    if(node){
        node->next = NULL;
        tal_system_memcpy(node,&input,sizeof(parallel_transfer_config_t));      
        while(head){
            if(head->next){
                head = head->next;   
            }
            else{
                break;
            }
        }
        if(head){
            head->next = node;
            result = TRUE;
        }
    }
    return result;
}


/**
 * @description: 将链表头即最远的一次发送数据取出，但是不破坏链表结构
 * @param {type} none
 * @return: none
 */
bool_t  get_transfer_list(parallel_transfer_config_t** output)
{
    bool_t result = FALSE;
    parallel_transfer_config_t *head = &g_file_head;
    bool_t have_push_flag = FALSE;

    while(head){
        if(head->next){
            head = head->next;
            have_push_flag = TRUE;
        }
        else{
            break;
        }
    }        
    if(have_push_flag){
        *output = head;
        result = TRUE;
    }
    return result;
}



/**
 * @description: 链表压出最远一次的发送数据
 * @param {type} none
 * @return: none
 */
bool_t  pop_transfer_list(void)
{
    bool_t result = FALSE;    
    parallel_transfer_config_t *last;
    parallel_transfer_config_t *head = &g_file_head;
    bool_t find_pre_node_flag = FALSE;

    while(head){
        last = head->next;
        if(last){            
            if(last->next){
                head = head->next;
                last = head->next;                
            }
            else{/*head->last->null,head是倒数第二个*/
                find_pre_node_flag = TRUE;
                break;
            }
        }
        else{ /*head就是最后一个*/
           break; 
        }
    }        
    if(find_pre_node_flag){
        if(head){
            if(last){
                tal_free(last);
            }
            head->next = NULL;
            result = TRUE;
        }
    }
    return result;
}



/**
 * @description: 确认是否需要停止传输中的数据，如果是TRUE，则需要将传输中的数据push起来；否则直接进行传输
 * @param {type} none
 * @return: none
 */
TRANSFER_STATUS_T check_stop_transfering_file(start_notification_pdu_t pdu,bool_t rec)
{
    UINT8_T input;
    UINT8_T receiving; 
    UINT8_T sending;
    TRANSFER_STATUS_T result;

    input = judge_transfer_cmd_type(pdu,rec);

    if((REC_TRANSFER_IDLE==g_run_info.rec.info.status) && (SEND_TRANSFER_IDLE==g_run_info.send.info.status)){
        if(rec){
            result = REC_IDLE;
            app_transfer_debug("REC_IDLE \r\n");
        }
        else{
            result = SEND_IDLE;
            app_transfer_debug("SEND_IDLE \r\n");
        }
    }
    else if((REC_TRANSFER_IDLE!=g_run_info.rec.info.status) && (SEND_TRANSFER_IDLE==g_run_info.send.info.status)){
        receiving = judge_transfer_cmd_type(g_run_info.rec.info.notify,TRUE);
        if(rec){/*正在接收时收到接收请求*/
            if(input < receiving){
                result = REC_STOP_REPLACE_REC;
                app_transfer_debug("REC_STOP_REPLACE_REC \r\n");
            }
            else{
                result = REC_BUSY;
                app_transfer_debug("REC_BUSY \r\n");
            }
        }
        else{/*正在接收时收到发送请求*/
            if (input < receiving){/*同步过程中接收上报红外码命令则将停止不同步，先进行上报*/
                result = REC_STOP_REPLACE_SEND;
                app_transfer_debug("REC_STOP_REPLACE_SEND \r\n");
            }
            else{
                result = SEND_IDLE;/*暂定其余状态上报和下载同步进行*/
                app_transfer_debug("SEND_IDLE \r\n");
            }
        }
    }
    else if((REC_TRANSFER_IDLE==g_run_info.rec.info.status) && (SEND_TRANSFER_IDLE!=g_run_info.send.info.status)){
        sending = judge_transfer_cmd_type(g_run_info.send.info.notify,FALSE);
        if(rec){/*正在发送时收到接收请求*/
            if(input < sending){
                result = SEND_STOP_REPLACE_REC;
                app_transfer_debug("SEND_STOP_REPLACE_REC \r\n");
            }
            else{
                result = REC_IDLE;/*暂定其余状态上报和下载同步进行*/
                app_transfer_debug("REC_IDLE \r\n");
            }
        }
        else{/*正在发送时收到发送请求*/
            if(input < sending){
                result = SEND_STOP_REPLACE_SEND;
                app_transfer_debug("SEND_STOP_REPLACE_SEND \r\n");
            }
            else{
                result = SEND_BUSY;/*暂定其余状态上报和下载同步进行*/
                app_transfer_debug("SEND_BUSY \r\n");
            }
        }
    }
    else{
        /*发送和接收可以同时进行的话，接收是控制，发送是上报*/
        receiving = judge_transfer_cmd_type(g_run_info.rec.info.notify,TRUE);
        sending = judge_transfer_cmd_type(g_run_info.send.info.notify,FALSE);
        if(rec){
            if(input < receiving){
                result = REC_SEND_STOP_REPLACE_REC;
                app_transfer_debug("REC_SEND_STOP_REPLACE_REC \r\n");
            }
            else{
                result = REC_BUSY;
                app_transfer_debug("REC_BUSY \r\n");
            }
        }
        else{
            if(input < sending){
                result = REC_SEND_STOP_REPLACE_SEND;
                app_transfer_debug("REC_SEND_STOP_REPLACE_SEND \r\n");
            }
            else{
                result = SEND_BUSY;/*暂定其余状态上报和下载同步进行*/
                app_transfer_debug("SEND_BUSY \r\n");
            }
        }
    }
    return result;
}




static void hal_multiple_packet_rec_evt_handle(TIMER_ID timer_id, VOID_T *arg)
{
    TAL_ZCL_MSG_T dev_msg;
    UINT32_T t_len=0;
    start_notification_pdu_t temp_pdu;
    TAL_MSG_RET_E result = ZCL_MSG_RET_FAILED;

    if(timer_id==sw_packet_callback_use){
        if(ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER == g_run_info.rec.info.notify.cmd_type)/*cluster下的command才会调用dev_msg_recv_callback*/
        {
            dev_msg.src_addr = g_run_info.rec.addr;
            dev_msg.cluster = (ZG_CLUSTER_ID_E)g_run_info.rec.info.notify.cluster_id;
            dev_msg.command = g_run_info.rec.info.notify.cmd;
            if(WHOLE_TYPE == g_run_info.rec.info.type){
                if(g_run_info.rec.info.notify.total_size>(g_run_info.rec.info.depth+MULTIPLE_PACKET_BUF_NEED)){/*防止越界*/
                    return;
                }
                else{
                    /*超过255字节的话，data的前4个字节是长度*/
                    if(g_run_info.rec.info.notify.total_size>=0xFF){                           
                        tal_system_memcpy(g_muliple_packet_rec_buf_head,(UINT8_T*)&g_run_info.rec.info.notify.total_size,sizeof(UINT32_T));
                        g_muliple_packet_rec_buf_head[sizeof(UINT32_T)+g_run_info.rec.info.notify.total_size] = '\0';
                        dev_msg.payload = g_muliple_packet_rec_buf_head;
                        dev_msg.length = 0xFF;
                    }
                    else{
                        dev_msg.length = g_run_info.rec.info.notify.total_size;
                        dev_msg.payload = g_run_info.rec.info.buf;
                    }
                }
            }
            else if(SUBSTEP_TYPE == g_run_info.rec.info.type){
                if(g_run_info.rec.info.depth>((g_run_info.rec.info.depth+MULTIPLE_PACKET_BUF_NEED)&0xFFFF)){/*防止越界*/
                    return;
                } 
                else{
                    /*最后一帧处理*/
                    if(g_run_info.rec.info.offset == g_run_info.rec.info.notify.total_size){
                        t_len = (g_run_info.rec.info.notify.total_size%g_run_info.rec.info.depth);
                    }
                    else{
                        t_len = g_run_info.rec.info.depth;
                    } 
                    /*超过255字节的话，data的前4个字节是长度*/
                    if(t_len>=0xFF){
                        tal_system_memcpy(g_muliple_packet_rec_buf_head,(UINT8_T*)&t_len,sizeof(UINT32_T));
                        dev_msg.payload = g_muliple_packet_rec_buf_head;
                        dev_msg.length = 0xFF;
                    }
                    else{
                        dev_msg.length = t_len;
                        dev_msg.payload = g_run_info.rec.info.buf;
                    }                         
                }                   
            }  

            result = tal_zcl_specific_msg_recv_callback(&dev_msg);

            ext_set_new_frame_flag(FALSE);

            if(SUBSTEP_TYPE == g_run_info.rec.info.type){/*应用层回调结束后将冗余多余的一帧数据转移到buf头部*/
                if(0xFF == dev_msg.length){
                    tal_system_memcpy(g_run_info.rec.info.buf,&g_run_info.rec.info.buf[g_run_info.rec.info.depth],\
                        g_run_info.rec.info.offset%g_run_info.rec.info.depth);
                    tal_system_memset(&g_run_info.rec.info.buf[g_run_info.rec.info.offset%g_run_info.rec.info.depth],\
                        0x00,((g_run_info.rec.info.depth+MULTIPLE_PACKET_BUF_NEED)-sizeof(UINT32_T)-g_run_info.rec.info.offset%g_run_info.rec.info.depth));    
                    tal_system_memset(g_muliple_packet_rec_buf_head,0x00,sizeof(UINT32_T));  
                    app_transfer_debug("deal\r\n");                 
                }
            }
            if(g_run_info.rec.info.offset == g_run_info.rec.info.notify.total_size){
                if(ZCL_MSG_RET_SUCCESS==result){
                    app_transfer_debug("download send true\r\n");
                    check_fileid_send_end_request(g_run_info.rec.info.notify.file_id,END_RES_SUCCESS); 
                }
                else{
                    app_transfer_debug("download send false\r\n");
                    check_fileid_send_end_request(g_run_info.rec.info.notify.file_id,END_RES_OTHER_ERROR);                    
                }                                        
                tal_system_memcpy((UINT8_T*)&temp_pdu,(UINT8_T*)&g_run_info.rec.info.notify,sizeof(start_notification_pdu_t));
                set_rec_wait_time(FALSE);
                refresh_transfer_rec_info(TRUE,TRUE);
                manage_transfering_file(temp_pdu,FALSE,TRUE,FALSE,FALSE);             
                app_transfer_debug("over\r\n");
            }
            else{
                if(SUBSTEP_TYPE==g_run_info.rec.info.type){/*应用层回调结束后主动发送block request帧*/
                    if(0xFF == dev_msg.length){
                        set_rec_wait_time(TRUE);
                        send_block_request_deal();  
                        app_transfer_debug("send block req\r\n");                   
                    }
                }
            }
        }
    }
}



/*************************************************************************************************
 * 
 * 
 *                                    分包接收部分
 * 
 * 
 * **************************************************************************************************/

/*收到start notify之后回调应用层让应用层进行数据申请，判断是否允许进行分包传输等*/
TUYA_WEAK_ATTRIBUTE bool_t app_start_notify_callback(UINT16_T addr,start_notification_pdu_t pdu,UINT8_T output_len,UINT8_T **input_buf,UINT16_T *tal_malloc_len)
{
    bool_t result = FALSE;
    return result;
}


/*分包结束回调，将对方信息和fileid上报给应用层，用于应用层释放start notify时申请的内存*/
TUYA_WEAK_ATTRIBUTE bool_t file_rec_transfer_end_callback(UINT16_T addr,UINT32_T file_id,bool_t result,UINT8_T *buf,UINT16_T len)
{
    return TRUE;
}


/**
 * @description: 将正在同步或是替换中的vir_id删除之后，需要发送end requests
 * @param {type} none
 * @return: none
 */
bool_t check_fileid_send_end_request(UINT16_T file_id,END_RESPOND_STATUS_T end_status)
{
    bool_t result = FALSE;
    if(file_id == g_run_info.rec.file_id){
        send_end_request_deal(end_status,TRUE);
        result = TRUE;
    }
    return result;
}



bool_t rec_new_frame_flag(void)
{
    return g_need_refresh_flag;
}


void ext_set_new_frame_flag(bool_t refresh_flag)
{
    g_need_refresh_flag = refresh_flag;        
}


/*数据接受完后回调应用层的接口函数，用于释放当前ram*/
static bool_t default_rec_msg_rec_callback(void)
{
    bool_t result = FALSE;

    tal_sw_timer_start(sw_packet_callback_use, 10, TAL_TIMER_ONCE);

    return result;
}


/*该函数功能是在app层开辟的ram接收数据满后，调用该函数通知应用层把数据进行保存，
之后app层会将该ram清空用于之后的数据接收，结束时就算不满也会调用一次*/
/*是否是最后一次调用，主要是区分大小*/
bool_t default_rec_substep_once_callback(bool_t first_step)
{
    if(first_step){
        ext_set_new_frame_flag(TRUE);
    }
    else{
        ext_set_new_frame_flag(FALSE);
    }
    return default_rec_msg_rec_callback();
}



static bool_t get_rec_transfer_type(MULIPLE_PACKET_TYPE_T *transfer_type)
{
    bool_t result = FALSE;

    *transfer_type = g_run_info.rec.info.type;
    result = TRUE;
    return result;
}



static bool_t refresh_transfer_rec_info(bool_t callback,bool_t result)
{
    bool_t return_result = TRUE;
    
    g_run_info.rec.addr=0xffff;
    g_run_info.rec.file_id = 0;
    g_run_info.rec.info.status = REC_TRANSFER_IDLE;
    g_run_info.rec.info.offset = 0;
    tal_system_memset((UINT8_T*)&g_run_info.rec.info.notify,0x00,sizeof(start_notification_pdu_t));
    if(TRUE==tal_sw_timer_is_running(sw_packet_callback_use)){
        tal_sw_timer_stop(sw_packet_callback_use);
    }
    if(callback){
        if(g_run_info.rec.info.buf && g_run_info.rec.info.depth){
            return_result = file_rec_transfer_end_callback(g_run_info.rec.addr,g_run_info.rec.file_id,result,g_run_info.rec.info.buf,(g_run_info.rec.info.depth+MULTIPLE_PACKET_BUF_NEED));
        }
    }
    app_transfer_debug("clear rec info \r\n");
    return return_result;
}


/*用于确认当前状态收到信息是否合法，return TRUE表示合法，可以进行下一步的判断；return FALSE表示非法，不需要下一步的判断*/
static REC_MULIPLE_PACKET_ERROR_CODE_T check_transfer_rec_addr_fileid_status(UINT16_T addr,start_notification_pdu_t pdu,MULTIPLE_PACKET_TRANSFER_CMD_TYPE_T cmd)
{
    REC_MULIPLE_PACKET_ERROR_CODE_T result = REC_NO_RESPOND;
    bool_t  match_flag = FALSE;

    if((g_run_info.rec.addr == addr) && (g_run_info.rec.file_id == pdu.file_id)){/*同一个设备及文件则判断自身状态及收到的帧类型*/
        match_flag = TRUE;
    }          
    if(match_flag){
        switch(g_run_info.rec.info.status){/*同一个传输过程中进行状态刷新*/
            case REC_TRANSFER_IDLE:  
                if(CMD_START == cmd){
                    app_transfer_debug("REC_START\r\n");
                    g_run_info.rec.info.status = REC_TRANSFER_START;
                    result = REC_SUCCESS_START;
                }
                else if(CMD_DOING == cmd){
                    app_transfer_debug("REC_DOING\r\n");
                    g_run_info.rec.info.status = REC_TRANSFER_DOING;
                    result = REC_SUCCESS_CONTINUE;
                }
                break;
            case REC_TRANSFER_START:
            case REC_TRANSFER_DOING: 
                if(CMD_DOING == cmd){
                    app_transfer_debug("REC_DOING\r\n");
                    g_run_info.rec.info.status = REC_TRANSFER_DOING;
                    result = REC_SUCCESS_CONTINUE;  
                }
                else if(CMD_END == cmd){
                    app_transfer_debug("REC_IDLE\r\n");
                    g_run_info.rec.info.status = REC_TRANSFER_IDLE;
                    result = REC_SUCCESS_END;  
                }
                break;
            default: 
                break;
        }
    }
    else{/*忙碌状态下收到了一次新的接收情况，需要进行接收数据管理*/
        if(CMD_START == cmd){
            if(manage_transfering_file(pdu,TRUE,FALSE,FALSE,FALSE)){/*start时是一定需要pdu输入的，start可能需要进行push*/  
                result = REC_SUCCESS_START; 
            }
            else{
                result = REC_FAIL_BUSY;
            }
        }
        else{
            result = REC_FAIL_BUSY;
        }
    }
    return result;
}



/*组件收到notify后进行的回调函数，由上层决定是否开启接收（错误码形式）*/
REC_MULIPLE_PACKET_ERROR_CODE_T app_rec_start_notify_callback(UINT16_T addr,start_notification_pdu_t pdu)
{
    REC_MULIPLE_PACKET_ERROR_CODE_T result = REC_FAIL_OTHER;
    UINT8_T *input_buf = NULL;
    UINT16_T tal_malloc_len = 0;

    result = check_transfer_rec_addr_fileid_status(addr,pdu,CMD_START);

    if(REC_SUCCESS_START == result)
    {
        if(app_start_notify_callback(addr,pdu,MULTIPLE_PACKET_BUF_MIN,&input_buf,&tal_malloc_len))
        {
            if(input_buf && (tal_malloc_len>MULTIPLE_PACKET_BUF_MIN)){
                g_muliple_packet_rec_buf_head = input_buf;
                g_run_info.rec.info.buf = (g_muliple_packet_rec_buf_head+sizeof(UINT32_T));    
                tal_system_memset(g_muliple_packet_rec_buf_head,0x00,tal_malloc_len);
                g_run_info.rec.info.depth = (tal_malloc_len-MULTIPLE_PACKET_BUF_NEED);
                result = REC_SUCCESS_START;
            }
            /*应用层传入的数据错误则直接返回end FALSE*/
            else{
                file_rec_transfer_end_callback(addr,pdu.file_id,FALSE,input_buf,tal_malloc_len);
            }
            if (REC_SUCCESS_START==result)
            {
                if(g_run_info.rec.info.depth<pdu.total_size){
                    g_run_info.rec.info.type = SUBSTEP_TYPE;
                }
                else{
                    g_run_info.rec.info.type = WHOLE_TYPE;
                }
                g_run_info.rec.addr = addr;
                g_run_info.rec.file_id = pdu.file_id;
                g_run_info.rec.info.status = REC_TRANSFER_START;
                g_run_info.rec.info.offset = 0;
                tal_system_memcpy((UINT8_T*)&g_run_info.rec.info.notify,(UINT8_T*)&pdu,sizeof(start_notification_pdu_t));
            }                 
        }            
    }
    return result;
}


bool_t  app_send_block_request_callback(UINT16_T* dest_addr,block_request_pdu_t* pdu)
{
    bool_t result = FALSE;

    app_transfer_debug("g_run_info.rec.info.status %d\r\n",g_run_info.rec.info.status);

    if(REC_TRANSFER_IDLE!=g_run_info.rec.info.status){
        *dest_addr = g_run_info.rec.addr;
        pdu->file_id = g_run_info.rec.file_id;
        pdu->file_offset = g_run_info.rec.info.offset;
        pdu->max_size = DEF_BLOCK_REQ_MAX_SIZE;
        result = TRUE;
    }
    return result;
}


/*需要分步进行的处理类型，每收到一帧调用一次*/
REC_MULIPLE_PACKET_ERROR_CODE_T app_rec_block_respond_callback(UINT16_T addr,UINT16_T file_id,bool_t check_result,UINT32_T offset,UINT8_T* rec_buf,UINT16_T length)
{
    REC_MULIPLE_PACKET_ERROR_CODE_T result = REC_NO_RESPOND;
    REC_MULIPLE_PACKET_ERROR_CODE_T check = REC_NO_RESPOND;
    MULIPLE_PACKET_TYPE_T transfer_type = WHOLE_TYPE;
    start_notification_pdu_t t_notify;

    app_transfer_debug("app_rec_block_respond_callback 1 check_result: %d,g_run_info.rec.info.offset: %d,offset: %d\r\n",check_result,g_run_info.rec.info.offset,offset);

    if(check_result && (g_run_info.rec.info.offset==offset))
    {  
        tal_system_memset((UINT8_T*)&t_notify,0x00,sizeof(start_notification_pdu_t));
        t_notify.file_id = file_id;
        check = check_transfer_rec_addr_fileid_status(addr,t_notify,CMD_DOING);
        app_transfer_debug("app_rec_block_respond_callback 2 check:%d\r\n",check);
        if(REC_SUCCESS_CONTINUE == check)
        {
            if(get_rec_transfer_type(&transfer_type))
            {
                switch(transfer_type){
                    case WHOLE_TYPE:
                        if((g_run_info.rec.info.offset+length) < (g_run_info.rec.info.notify.total_size)){
                            tal_system_memcpy((UINT8_T*)&g_run_info.rec.info.buf[g_run_info.rec.info.offset],rec_buf,length);
                            g_run_info.rec.info.offset+=length;
                            result = REC_SUCCESS_CONTINUE;
                            app_transfer_debug("app_rec_block_respond_callback 3\r\n");
                        }
                        else if((g_run_info.rec.info.offset+length) == (g_run_info.rec.info.notify.total_size)){
                            tal_system_memcpy((UINT8_T*)&g_run_info.rec.info.buf[g_run_info.rec.info.offset],rec_buf,length);
                            g_run_info.rec.info.offset+=length;
                            ext_set_new_frame_flag(TRUE);/*一次性分包*/
                            result = REC_SUCCESS_END;/*底层会调用一次app_multiple_packet_end_callback*/
                            app_transfer_debug("app_rec_block_respond_callback 4\r\n");
                        }
                    break;
                    case SUBSTEP_TYPE:
                        if(TRUE==tal_sw_timer_is_running(sw_packet_callback_use)){
                            result = REC_FAIL_OTHER;/*等待回调时需要接收到数据不能放到buf里面*/
                        }
                        else{
                            if((g_run_info.rec.info.offset+length) < (g_run_info.rec.info.notify.total_size)){
                                result = REC_SUCCESS_CONTINUE;
                            }
                            else if((g_run_info.rec.info.offset+length) == (g_run_info.rec.info.notify.total_size)){
                                result = REC_SUCCESS_END;/*底层会调用一次app_multiple_packet_end_callback*/
                            }
                            
                            if((REC_SUCCESS_END==result)||(REC_SUCCESS_CONTINUE==result)){

                                tal_system_memcpy((UINT8_T*)&g_run_info.rec.info.buf[g_run_info.rec.info.offset%g_run_info.rec.info.depth],rec_buf,length);
                                
                                /*分步传输类型需要进行一次应用数据回调，并清除一步的数据；数据传输完成则将剩余的所有数据进行应用回调*/
                                if(((g_run_info.rec.info.offset+length)/g_run_info.rec.info.depth) && (((g_run_info.rec.info.offset+length)%g_run_info.rec.info.depth)<length))/*这里不能==因为push出来就是2048的整数倍*/
                                {
                                    /*避免凑巧情况下，分步过程中调了一次app callback且end 也调用了一次app callback*/
                                    /*分步的最后一次调用，不应该在这里收到之后立即调用，而需要在end 部分处理*/
                                    if(!(REC_SUCCESS_END == result))
                                    {             
                                        app_transfer_debug("once buf rec ok and recallback\r\n");             
                                        if(g_run_info.rec.info.offset<=g_run_info.rec.info.depth){
                                            g_run_info.rec.info.offset += length;
                                            default_rec_substep_once_callback(TRUE); 
                                        }
                                        else{
                                            g_run_info.rec.info.offset += length;
                                            default_rec_substep_once_callback(FALSE);                                
                                        }
                                    }
                                }
                                else{
                                    g_run_info.rec.info.offset += length;
                                }
                            }
                        }
                    break;
                    default:
                    break;
                }
            }
        }
    }
    app_transfer_debug("rec block res: %d",result);
    return result;
}

/*要求是根据应用层的返回值来决定是否发送end request，底层发送是默认不能主动发送end request的*/
bool_t  app_send_end_request_callback(UINT16_T* dest_addr,end_request_pdu_t* pdu,END_RESPOND_STATUS_T end_status,bool_t force)
{
    bool_t result = FALSE;
    start_notification_pdu_t temp_pdu = g_run_info.rec.info.notify;

    end_send_try_times();
    *dest_addr = g_run_info.rec.addr;
    pdu->end_status = end_status;
    pdu->file_id = g_run_info.rec.file_id;
    pdu->reserved =  0;
    
    if(END_RES_SUCCESS == end_status){/*接收成功后回调应用层*/
        result = default_rec_msg_rec_callback();
    }
    else{/*如果接收失败则进行check,确认是否有被暂停的任务*/
        refresh_transfer_rec_info(TRUE,FALSE);
        /*确认是否处于同步替换状态将下载过程中的数据清除*/
        manage_transfering_file(temp_pdu,FALSE,TRUE,FALSE,FALSE); 
    }
    if(force){
        result = TRUE;
    }
    return result;
}


/*完成接收后进行的调用，分步的方式不需要再传入数据，直接传入NULL，len=0就可以*/
REC_MULIPLE_PACKET_ERROR_CODE_T app_rec_end_respond_callback(UINT16_T addr,UINT16_T file_id)
{
    REC_MULIPLE_PACKET_ERROR_CODE_T result = REC_FAIL_OTHER;
    start_notification_pdu_t t_notify;

    tal_system_memset((UINT8_T*)&t_notify,0x00,sizeof(start_notification_pdu_t));
    t_notify.file_id = file_id;

    result = check_transfer_rec_addr_fileid_status(addr,t_notify,CMD_END);

    return result;
}






/*************************************************************************************************
 * 
 * 
 *                                    分包发送部分
 * 
 * 
 * **************************************************************************************************/


TUYA_WEAK_ATTRIBUTE bool_t file_send_transfer_end_callback(UINT16_T addr,UINT32_T file_id,bool_t result,UINT8_T *buf,UINT16_T len)
{
    return TRUE;
}



/*该函数功能是在app层开辟的ram发送完毕后清除发送过的数据再调用该函数，重新载入需要发送的数据进行发送*/
TUYA_WEAK_ATTRIBUTE bool_t default_send_substep_once_callback(start_notification_pdu_t pdu,UINT32_T offset,UINT8_T*data,UINT16_T *len)
{
    bool_t result = FALSE;

    if((pdu.total_size>offset) && (g_run_info.send.info.depth>=*len) && (NULL!=data)){
        switch(pdu.cluster_id)
        {
#if 0
            case CLUSTER_TUYA_IR_CONTROL_CLUSTER_ID:
                /*进行数据装载，len为本次状态的数据长度*/
                result = TRUE;
            break;
#endif
            default:
            break;
        }
    }
    return result;
}


/*红外发送成功之后的回调函数，用于清除状态机及发送的数据*/
bool_t stop_send_stop_block_respond(void)
{
    return refresh_transfer_send_info(TRUE,FALSE);
}


/*获取主动发起分包的file_id*/
static UINT16_T get_muliple_packet_send_file_id(void)
{
    if(g_muliple_packet_send_file_id == 0xffff){
        g_muliple_packet_send_file_id = 0;
    }
    else{
        g_muliple_packet_send_file_id++;
    }
    return g_muliple_packet_send_file_id;
}


/*初始化发送分包数据的状态及数据*/
static bool_t refresh_transfer_send_info(bool_t callback,bool_t result)
{
    bool_t return_result = TRUE;

    g_run_info.send.addr=0xffff;
    g_run_info.send.file_id = 0;
    g_run_info.send.info.status = SEND_TRANSFER_IDLE;
    g_run_info.send.info.offset = 0;
    tal_system_memset((UINT8_T*)&g_run_info.send.info.notify,0x00,sizeof(start_notification_pdu_t));
    if(callback){
        if(g_run_info.send.info.buf && g_run_info.send.info.depth){
            result = file_send_transfer_end_callback(g_run_info.send.addr,g_run_info.send.file_id,result,g_run_info.send.info.buf,g_run_info.send.info.depth);
        }
    }    
    app_transfer_debug("clear send info \r\n");
    return return_result;
}


/*用于确认当前状态收到信息是否合法，return TRUE表示合法，可以进行下一步的判断；return FALSE表示非法，不需要下一步的判断*/
static bool_t check_transfer_send_addr_fileid_status(UINT16_T addr,start_notification_pdu_t pdu,MULTIPLE_PACKET_TRANSFER_CMD_TYPE_T cmd)
{
    bool_t result = FALSE;
    bool_t  match_flag = FALSE;

    if((g_run_info.send.addr == addr) && (g_run_info.send.file_id == pdu.file_id)){/*同一个设备及文件则判断自身状态及收到的帧类型*/
        match_flag = TRUE;
    }           
    if(match_flag){
        switch(g_run_info.send.info.status){/*同一个传输过程中进行状态刷新*/
            case SEND_TRANSFER_IDLE:  
                if(CMD_START == cmd){
                    app_transfer_debug("change status to SEND_START\r\n");
                    g_run_info.send.info.status = SEND_TRANSFER_START;
                    result = TRUE;
                }
                break;
            case SEND_TRANSFER_START:
            case SEND_TRANSFER_DOING: 
                if(CMD_DOING == cmd){
                    app_transfer_debug("change status to SEND_DOING\r\n");
                    g_run_info.send.info.status = SEND_TRANSFER_DOING;
                    result = TRUE;  
                }
                else if(CMD_END == cmd){
                    app_transfer_debug("change status to SEND_IDLE\r\n");
                    manage_transfering_file(pdu,FALSE,FALSE,FALSE,TRUE);   
                    g_run_info.send.info.status = SEND_TRANSFER_IDLE;
                    result = TRUE;  
                }
                break;
            default: 
                break;
        }
    }
    else if(CMD_START == cmd){
        manage_transfering_file(pdu,FALSE,FALSE,TRUE,FALSE); 
    }
    return result;
}


/*主动发送数据的入口函数*/
bool_t app_start_send_multiple_packet_handle(UINT16_T addr,UINT16_T cluster,UINT8_T type,UINT8_T cmd,UINT32_T total_size,UINT8_T* data,UINT16_T len,UINT16_T *file_id)
{
    bool_t result = TRUE;
    start_notification_pdu_t *pdu = (start_notification_pdu_t *)tal_malloc(sizeof(start_notification_pdu_t));

    if(pdu){
        if((!total_size)||(!len)||(NULL==data)){
            tal_free(pdu);
            return FALSE;
        }
        if(SEND_TRANSFER_IDLE!=g_run_info.send.info.status){
            result = file_send_transfer_end_callback(g_run_info.send.addr,g_run_info.send.file_id,FALSE,g_run_info.send.info.buf,g_run_info.send.info.depth);
        }
        if(result)
        {
            if(refresh_transfer_send_info(FALSE,FALSE)){

                pdu->file_id = get_muliple_packet_send_file_id();
                pdu->total_size = total_size;
                pdu->random_time = 0;
                pdu->cluster_id = cluster;
                pdu->cmd_type = type;
                pdu->cmd = cmd;
                pdu->reserved = 0;
                tal_system_memcpy((UINT8_T*)&g_run_info.send.info.notify,(UINT8_T*)pdu,sizeof(start_notification_pdu_t));

                g_run_info.send.addr = addr;
                g_run_info.send.file_id = pdu->file_id;
                g_run_info.send.info.status = SEND_TRANSFER_IDLE;
                g_run_info.send.info.buf = data;                    
                g_run_info.send.info.depth = len; 

                if(total_size <= len){                
                    g_run_info.send.info.type = WHOLE_TYPE;
                }
                else {
                    g_run_info.send.info.type = SUBSTEP_TYPE;
                }

                g_run_info.send.info.offset = 0;
                send_start_notify_deal(addr,*pdu); 
                result = TRUE;               
            }
        }
        tal_free(pdu);
    }
    return result;
}


/*对方发送start notify respond后传入app进行数据刷新*/
SEND_MULIPLE_PACKET_ERROR_CODE_T app_rec_notify_respond_callback(UINT16_T addr,start_respond_pdu_t pdu)
{
    SEND_MULIPLE_PACKET_ERROR_CODE_T result = SEND_FAIL_OTHER;
    start_notification_pdu_t temp_notify;

    tal_system_memcpy((UINT8_T*)&temp_notify,(UINT8_T*)&pdu.file_id,sizeof(start_notification_pdu_t));
    if(check_transfer_send_addr_fileid_status(addr,temp_notify,CMD_START)){
        if(0 == tal_system_memcmp((char const*)&g_run_info.send.info.notify,(char const*)&temp_notify,sizeof(start_notification_pdu_t)))
        {
            g_run_info.send.info.status = SEND_TRANSFER_START;
            result = SEND_NOTIFY_SUCCESS;  
        }
    }
    return result;
}



/*hal层获取到对方设备发来的block requeset后传给app层进行状态同步及数据刷新，并根据对方获取的格式及offset决定是否进行发送block respond*/
SEND_MULIPLE_PACKET_ERROR_CODE_T app_rec_block_requset_callback(UINT16_T dest_addr,block_request_pdu_t pdu)
{
    SEND_MULIPLE_PACKET_ERROR_CODE_T result = SEND_FAIL_OTHER;
    start_notification_pdu_t temp_notify;

    tal_system_memset((UINT8_T*)&temp_notify,0x00,sizeof(start_notification_pdu_t));
    temp_notify.file_id = pdu.file_id;

    if(check_transfer_send_addr_fileid_status(dest_addr,temp_notify,CMD_DOING))
    {
        if((pdu.file_offset<g_run_info.send.info.notify.total_size) && (pdu.max_size<=DEF_BLOCK_REQ_MAX_SIZE) && (pdu.max_size)) {

            if(pdu.max_size>=(DEF_BLOCK_REQ_MAX_SIZE-BLOCK_RESPOND_FORMAT_PLAYLOAD_LEN)){
                g_run_info.send.info.packet_size = (DEF_BLOCK_REQ_MAX_SIZE-BLOCK_RESPOND_FORMAT_PLAYLOAD_LEN);
            }
            else{
                g_run_info.send.info.packet_size = pdu.max_size;
            }
            g_run_info.send.info.offset = pdu.file_offset;
            result = SEND_BLOCK_SUCCESS;
        }
    }
    return result;
}


/*获取应用层维护的当前需要进行发送的block respond的pdu,串行分包版本*/
bool_t app_send_block_respond_callback(UINT16_T* dest_addr,block_respond_pdu_t* pdu)
{
    bool_t result = FALSE;

    if(SEND_TRANSFER_DOING==g_run_info.send.info.status)
    {
        switch(g_run_info.send.info.type)
        {
            case WHOLE_TYPE:
                if((g_run_info.send.info.offset+g_run_info.send.info.packet_size)<=g_run_info.send.info.notify.total_size){
                    pdu->length = g_run_info.send.info.packet_size;
                    pdu->data = &g_run_info.send.info.buf[g_run_info.send.info.offset];
                    result = TRUE;
                }
                else{
                    pdu->length = g_run_info.send.info.notify.total_size-g_run_info.send.info.offset;
                    pdu->data = &g_run_info.send.info.buf[g_run_info.send.info.offset];
                    result = TRUE;
                }
            break;
            case SUBSTEP_TYPE:
                if((g_run_info.send.info.depth) && (g_run_info.send.info.packet_size<=g_run_info.send.info.depth))/*暂时不考虑depth小于一帧状态的数据长度，因为需要多次装载*/
                {
                    if((!(g_run_info.send.info.offset%g_run_info.send.info.depth)) && (g_run_info.send.info.offset<g_run_info.send.info.notify.total_size)){
                        default_send_substep_once_callback(g_run_info.send.info.notify,g_run_info.send.info.offset,g_run_info.send.info.buf,&g_run_info.send.info.depth);
                    }
                    if(((g_run_info.send.info.offset%g_run_info.send.info.depth)+g_run_info.send.info.packet_size)<=g_run_info.send.info.depth){
                        pdu->length = g_run_info.send.info.packet_size;
                        pdu->data =  &g_run_info.send.info.buf[g_run_info.send.info.offset%g_run_info.send.info.depth];
                        result = TRUE;
                    }
                    else{
                        pdu->length = g_run_info.send.info.depth-(g_run_info.send.info.offset%g_run_info.send.info.depth);
                        pdu->data =  &g_run_info.send.info.buf[g_run_info.send.info.offset%g_run_info.send.info.depth];
                        result = TRUE;                             
                    }
                }
            break;
            default:
            break;
        }    
        if(TRUE == result){
            *dest_addr = g_run_info.send.addr;
            pdu->block_status = BLOCK_RES_SUCCESS;
            pdu->file_id = g_run_info.send.file_id;
            pdu->file_offset = g_run_info.send.info.offset;
            pdu->checksum = u8_checksum(pdu->data,pdu->length);
        }
    }
    return result;
}


/*hal层获取到对方设备发来的end requeset后将数据传给app层进行状态同步及数据刷新*/
SEND_MULIPLE_PACKET_ERROR_CODE_T  app_rec_end_request_callback(UINT16_T dest_addr,end_request_pdu_t pdu)
{
    SEND_MULIPLE_PACKET_ERROR_CODE_T result = SEND_FAIL_OTHER;
    start_notification_pdu_t temp_notify;

    tal_system_memset((UINT8_T*)&temp_notify,0x00,sizeof(start_notification_pdu_t));
    temp_notify.file_id = pdu.file_id;

    if(check_transfer_send_addr_fileid_status(dest_addr,temp_notify,CMD_END)){
        if(END_RES_SUCCESS==pdu.end_status){
            refresh_transfer_send_info(TRUE,TRUE);
        }
        else{
            refresh_transfer_send_info(TRUE,FALSE);
        }
        result = SEND_END_SUCCESS;  
    }
    return result;
}




bool_t tdd_file_transfer_init(bool_t rec_enable,bool_t send_enable,bool_t fast_poll,bool_t rec_timeout,bool_t send_timeout)
{
    bool_t result = FALSE;
    muliple_packet_config_t config;

    if((!rec_enable) && (!send_enable)){
        return FALSE;
    }

    if(((!rec_enable) && rec_timeout) || ((!send_enable) && send_timeout)){
        return FALSE;        
    }

    if(rec_enable){
        config.rec_notify_req = app_rec_start_notify_callback;
        config.rec_block_res = app_rec_block_respond_callback;
        config.rec_end_res = app_rec_end_respond_callback;
        config.send_block_req = app_send_block_request_callback;
        config.send_end_req = app_send_end_request_callback;
        refresh_transfer_rec_info(FALSE,FALSE);
    }
    else{
        config.rec_notify_req = NULL;
        config.rec_block_res = NULL;
        config.rec_end_res = NULL;
        config.send_block_req = NULL;
        config.send_end_req = NULL;
    }

    if(send_enable){
        config.rec_notify_res = app_rec_notify_respond_callback;
        config.rec_block_req = app_rec_block_requset_callback;
        config.send_block_res = app_send_block_respond_callback;
        config.rec_end_req = app_rec_end_request_callback;
        refresh_transfer_send_info(FALSE,FALSE);
    }
    else{
        config.rec_notify_res = NULL;
        config.rec_block_req = NULL;
        config.send_block_res = NULL;
        config.rec_end_req = NULL; 
    }

    if(rec_enable||send_enable){

        if(OPRT_OK==tal_sw_timer_create(hal_multiple_packet_rec_evt_handle, NULL, &sw_packet_callback_use)){
            config.send_result = app_send_data_result_callback;
            multiple_packet_transfer_init(config,fast_poll,rec_timeout,send_timeout);   
            result = TRUE;          
        }

    }
    return result;
}


