/**
* @file app_common.c
* @brief Common process - adapter the flash api
* @version 0.1
* @date 2021-10-22
*
* @copyright Copyright 2020-2021 Tuya Inc. All Rights Reserved.
*
*/
#include "tkl_pwm.h"
#include "tkl_network_manager.h"

#include "tal_system.h"
#include "tal_memory.h"
#include "tal_log.h"
#include "tal_pwm.h"
#include "tal_gpio.h"
#include "tal_uart.h"
#include "tal_flash.h"
#include "tal_sw_timer.h"
#include "tal_attribute_rw.h"
#include "app_common.h"
#include "tal_nv_flash.h"
#include "tdd_file_transfer.h"
#include "tal_network_mgr.h"


typedef struct {
    BOOL_T enabled;
    BOOL_T driver_flag;
    TUYA_GPIO_NUM_E pin_num;
} OEM_LED_INFO_T;

//led info
typedef struct {
    OEM_LED_INFO_T status_led;
} APP_LED_INFO_T;

#define KEY_SCAN_MAX_CNT        250

extern TIMER_ID etimer_key_scan;
extern TIMER_ID etimer_join_start_delay;
extern TIMER_ID etimer_led_blink_event_id;

STATIC UINT16_T sg_key_scan_cnt = 0;
STATIC BOOL_T g_status_led_onoff_flag = FALSE;

UINT8_T *ir_compress_send_record = NULL;
UINT32_T ir_compress_send_len = (sizeof(UINT8_T))*2*1024;


/**
 * @brief 
 * 
 * @param str 
 * @return VOID_T 
 */
VOID_T dev_uart_output(IN CONST CHAR_T *str)
{
    tal_uart_write((TUYA_UART_NUM_E)USER_UART0, (CONST UINT8_T *)str, strlen(str));
}
/**
 * @brief 
 * 
 * @param args 
 * @return VOID_T 
 */
STATIC VOID_T __dev_pin_irq_handle(VOID_T *args)
{
    sg_key_scan_cnt = 0;
    tal_sw_timer_start(etimer_key_scan, 20, TAL_TIMER_ONCE);
}

/**
 * @brief 
 * 
 * @return VOID_T 
 */
VOID_T app_button_init(VOID_T)
{
    TUYA_GPIO_IRQ_T irq_cfg = {
        .mode = TUYA_GPIO_IRQ_FALL,
        .cb = __dev_pin_irq_handle,
        .arg = NULL,
    };
    tal_gpio_irq_init(USER_KEY0, &irq_cfg);//PC0
}
/**
 * @brief 
 * 
 * @param timer_id 
 * @param arg 
 * @return VOID_T 
 */
VOID_T app_button_scan_cb(TIMER_ID timer_id, VOID_T *arg)
{
    TUYA_GPIO_LEVEL_E v_level = TUYA_GPIO_LEVEL_HIGH;
    UINT16_T file_id = 0;
    TAL_ZG_NWK_STATUS_E net_states = tal_zg_nwk_status_get();

    if (OPRT_OK != tal_gpio_read(USER_KEY0, &v_level))
    {
        return;
    }

    if (v_level == TUYA_GPIO_LEVEL_HIGH)
    {
        if(sg_key_scan_cnt < KEY_SCAN_MAX_CNT){  
            if(TAL_ZG_NWK_JOIN_START != net_states){
                if(ir_compress_send_record == NULL){
                    ir_compress_send_record = tal_malloc(ir_compress_send_len);
                }
                if(ir_compress_send_record){
                    tal_system_memset(ir_compress_send_record,0xFF,ir_compress_send_len);
                    app_start_send_multiple_packet_handle(0x0000,CLUSTER_TUYA_IR_CONTROL_CLUSTER_ID,ZG_ZCL_FRAME_TYPE_SPEC_TO_CLUSTER,\
                        CMD_IR_STUDY_KEYCODE_COMMAND_ID,ir_compress_send_len,ir_compress_send_record,ir_compress_send_len,&file_id);
                }
            }
        }
        sg_key_scan_cnt = 0;
    }
    else
    {
        sg_key_scan_cnt++;
        if (sg_key_scan_cnt >= KEY_SCAN_MAX_CNT)
        {
            tal_sw_timer_start(etimer_join_start_delay, 50, TAL_TIMER_ONCE);
            return;
        }
        tal_sw_timer_start(etimer_key_scan, 20, TAL_TIMER_ONCE);
    }
}


/**
 * @description: set status led on off state
 * @param[in] state: on off state
 * @return: none
 */
STATIC VOID_T status_led_set(BOOL_T onoff)
{
    if (onoff) {
        tal_gpio_write(NET_STATUS_LED0, TUYA_GPIO_LEVEL_HIGH);
    } else {
        tal_gpio_write(NET_STATUS_LED0, TUYA_GPIO_LEVEL_LOW);
    } 
}


/**
 * @description: stop led blink
 * @param[in] none
 * @return: none
 */
VOID_T status_led_stop_blink(BOOL_T onoff)
{
    tal_sw_timer_stop(etimer_led_blink_event_id);
    status_led_set(onoff);
    g_status_led_onoff_flag = onoff;
}


VOID_T status_led_start_blink(BOOL_T onoff)
{
    tal_sw_timer_stop(etimer_led_blink_event_id);
    status_led_set(onoff);
    g_status_led_onoff_flag = onoff;
    tal_sw_timer_start(etimer_led_blink_event_id, 250, TAL_TIMER_ONCE);
}

/**
 * @description: led blink event
 * @param [in] timer_id: timer id
 * @param[in] args: null
 * @return: none
 */
STATIC VOID_T app_led_blink_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    status_led_set(g_status_led_onoff_flag);
    g_status_led_onoff_flag = (!g_status_led_onoff_flag);
    tal_sw_timer_start(etimer_led_blink_event_id, 250, TAL_TIMER_ONCE);
}

/**
 * @description: led blink event create
 * @param[in] none
 * @return: none
 */
VOID_T app_status_led_init(VOID_T)
{
    TUYA_GPIO_BASE_CFG_T status_config;

    tal_system_memset(&status_config, 0, SIZEOF(TUYA_GPIO_BASE_CFG_T));
    status_config.mode = TUYA_GPIO_PUSH_PULL;
    status_config.direct = TUYA_GPIO_OUTPUT;
    status_config.level = TUYA_GPIO_LEVEL_HIGH;
    tal_gpio_init(NET_STATUS_LED0, &status_config);

    tal_sw_timer_create(app_led_blink_event_handler, NULL, &etimer_led_blink_event_id);
}